
var Plate = function()
{
	var plate = {};
	plate.rows = 8;
	plate.columns = 12;
	plate.wells = [];
	plate.poolSchema = null;
	plate.wellPainter = null;
	
	/**
		Initialize a plate with the given number of rows and columns.
	*/
	plate.init = function(rows, columns, poolSchema, wellPainter)
	{
		if (rows) plate.rows = rows;
		if (columns) plate.columns = columns;
		if (poolSchema) plate.poolSchema = poolSchema;
		if (wellPainter) plate.wellPainter = wellPainter;
		
		for (var c = 0; c < plate.columns; c++)
		{
			for (var r = 0; r < plate.rows; r++)
			{
				plate.wells[plate.wells.length] = new Well(r, c);
			}
		}
	}
	
	/**
		Change the pool layout schema for the plate.
	*/
	plate.setPoolSchema = function(poolSchema)
	{
		plate.poolSchema = poolSchema;
	}

	/**
		Change the well painter for the plate.
	*/
	plate.setWellPainter = function(wellPainter)
	{
		plate.wellPainter = wellPainter;
	}
	
	/**
		Get the number of rows on the plate.
	*/
	plate.getRows = function()
	{
		return plate.rows;
	}

	/**
		Get the number of columns on the plate.
	*/
	plate.getColumns = function()
	{
		return plate.columns;
	}

	/**
		Get the number of pools on the plate.
		If no schema has been set, 0 is returned.
	*/
	plate.getPools = function()
	{
		return plate.poolSchema ? plate.poolSchema.numPools : 0;
	}
	
	/**
		Get all wells on the plate.
	*/
	plate.getWells = function()
	{
		return plate.wells;
	}
	
	/**
		Get all wells in the given row. First row is 0.
	*/
	plate.getRow = function(row)
	{
		var result = [];
		for (var i = row; result.length < plate.columns; i += plate.rows)
		{
			result[result.length] = plate.wells[i];
		}
		return result;
	}
	
	/**
		Get all wells in the given column. First column is 0.
	*/
	plate.getColumn = function(column)
	{
		var result = [];
		for (var i = column*plate.rows; result.length < plate.rows; i++)
		{
			result[result.length] = plate.wells[i];
		}
		return result;
	}

	
	/**
		Get all wells in the given pool number. Which wells are selected,
		depends on the pool schema. First pool number is 0.
	*/
	plate.getPool = function(poolNum)
	{
		return plate.poolSchema.getWellsInPool(plate, poolNum);
	}

	/**
		Get the well at the given coordinate.
	*/
	plate.getWell = function(row, column)
	{
		return plate.wells[row + column * plate.rows];
	}
	
	/**
		Get all wells that contains extract with a given name.
	*/
	plate.getWellsByName = function(name)
	{
		var result = [];
		for (var i = 0; i < plate.wells.length; i++)
		{
			var well = plate.wells[i];
			if (well.extract && well.extract.name == name)
			{
				result[result.length] = plate.wells[i];
			}
		}
		return result;
	}
	
	/**
		Get all wells that are selected.
	*/
	plate.getSelected = function()
	{
		var selected = [];
		for (var i = 0; i < plate.wells.length; i++)
		{
			var well = plate.wells[i];
			if (well.selected)
			{
				selected[selected.length] = well;
			}
		}
		return selected;
	}

	/**
		Set the selected status for all wells in the array.
		The wells are automatically repainted.
	*/
	plate.setSelected = function(wells, select)
	{
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			well.selected = select;
			well.paint(plate.wellPainter, plate.poolSchema);
		}
	}
	
	/**
		Toggle the selected status for all wells in the array.
		The next status is determined by the current status of 
		the first well in the array.
	*/
	plate.toggleSelected = function(wells)
	{
		if (wells.length == 0) return;
		plate.setSelected(wells, !wells[0].selected);
	}
	
	/**
		Enable or disable highlight of the given wells.
	*/
	plate.setHighlight = function(wells, highlightClass, on)
	{
		var tmpClass = on ? highlightClass : null;
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			well.highlightClass = tmpClass;
			Main.addOrRemoveClass(well.tag, highlightClass, on);
		}
	}
	
	/**
		Paint the given wells.
	*/
	plate.paint = function(wells)
	{
		for (var i = 0; i < wells.length; i++)
		{
			wells[i].paint(plate.wellPainter, plate.poolSchema);
		}
	}
	
	/**
		Check if the plate have any replicated extracts. Eg. extracts that are
		found in more than one well. If a name is given, only extracts with that
		name is checked, otherwise the complete plate is checked. The
		result of the check is stored in the 'replicate' flag of the well.
	*/
	plate.checkReplicates = function(name, noRepaint)
	{
		var result = [];
		if (name)
		{
			// If a name has been given, it is relatively easy
			var wells = plate.getWellsByName(name);
			var isReplicate = wells.length > 1;
			for (var i = 0; i < wells.length; i++)
			{
				var well = wells[i];
				if (well.replicate != isReplicate)
				{
					well.replicate = isReplicate;
					if (!noRepaint) well.paint(plate.wellPainter, plate.poolSchema);
				}
			}
		}
		else
		{
			// No name, we begin by counting the number of times we find any given name
			var nameCount = [];
			for (var i = 0; i < plate.wells.length; i++)
			{
				var well = plate.wells[i];
				// Ignore empty wells and wells with 'Stratagene' or external 
				if (well.extract && !well.extract.stratagene && !well.extract.external)
				{
					nameCount[well.extract.name] = 1 + (nameCount[well.extract.name] || 0);
				}
			}
			// Then, we update the status for all wells 
			for (var i = 0; i < plate.wells.length; i++)
			{
				var well = plate.wells[i];
				if (well.extract)
				{
					var isReplicate = nameCount[well.extract.name] > 1;
					if (well.replicate != isReplicate)
					{
						well.replicate = isReplicate;
						if (!noRepaint) well.paint(plate.wellPainter, plate.poolSchema);
					}
				}
			}
		}
	}
	
	return plate;
}();

/**
	Represents a well on the plate. Each well is created when 
	the page is loaded and is initially empty.
*/
function Well(row, column)
{
	this.row = row;
	this.column = column;
	this.selected = false;
	this.highlighClass = null;
	this.extract = null;
	this.duplicates = null;
	this.replicate = false;
	this.warning = null;
	this.error = null;
	
	this.tag = document.getElementById('well.'+row+'.'+column);
}

/**
	Set the extract item that is put into this well. Use
	'null' to clear the well. The extract should always
	have a 'name' property, and an 'id' property if it 
	exists in the database.
*/
Well.prototype.setExtract = function(extract)
{
	this.extract = extract;
	this.duplicates = null;
	this.replicate = false;
	if (extract && this.clipboard) this.doneWithCopy();
}

/**
	Set duplicate extract items in this well. This can
	happen when using a file import if the same location
	is specified twice and is an error condition which
	must be resolved.
*/
Well.prototype.addDuplicate = function(extract)
{
	if (!this.duplicates)
	{
		this.duplicates = [];
		this.duplicates[0] = this.extract.name;
	}
	this.duplicates[this.duplicates.length] = extract.name;
}

/**
	Create a copy-object holding information about this well.
	The copy has 'well' property pointing to this well and
	'name' property holding the extract name if the well is not
	empty. The 'clipboard' flag of this well is set to true.
*/
Well.prototype.makeCopy = function()
{
	var cp = {};
	cp.well = this;
	cp.name = this.extract ? this.extract.name : null;
	cp.comment = this.extract ? this.extract.comment : null;
	this.clipboard = true;
	this.copyText = cp.name;
	return cp;
}

/**
	Unset the 'clipboard' flag for this well.
*/
Well.prototype.doneWithCopy = function()
{
	this.clipboard = false;
	this.copyText = null;
}

Well.prototype.setWarning = function(warning)
{
	this.warning = warning;
}

Well.prototype.hasWarning = function()
{
	return this.warning != null;
}

Well.prototype.setError = function(error)
{
	this.error = error;
}

Well.prototype.hasError = function()
{
	return this.error != null;
}

Well.prototype.getClassName = function(wellPainter, poolSchema)
{
	var c = this.column;
	var cls = 'well col-'+c;
	if (this.selected) cls += ' selected';
	if (this.clipboard) cls += ' clipboard';
	if (this.hasError()) cls += ' err';
	if (this.hasWarning()) cls += ' warning';
	if (this.highlightClass) cls += ' ' + this.highlightClass;

	if (poolSchema)
	{
		cls += ' ' + poolSchema.getClassNameForWell(this, wellPainter);
	}
	if (wellPainter)
	{
		cls += ' ' + wellPainter.getClassNameForWell(this, poolSchema);
	}
	return cls;
}

Well.prototype.getText = function(wellPainter, poolSchema)
{
	var text = '';
	if (wellPainter)
	{
		text = wellPainter.getWellText(this, poolSchema);
	}
	else
	{
		if (this.extract)
		{
			text += '<div class="name">'+this.extract.name+'</div>'
		}
	}
	if (this.hasError())
	{
		text += '<div class="err-msg">'+this.error + '</div>';
	}
	if (this.hasWarning())
	{
		text += '<div class="warn-msg">'+this.warning + '</div>';
	}
	return text;
}

Well.prototype.paint = function(wellPainter, poolSchema)
{
	this.warning = null;
	this.error = null;
	this.tag.innerHTML = this.getText(wellPainter, poolSchema);
	this.tag.className = this.getClassName(wellPainter, poolSchema);
}


